@@ -416,26 +416,9 @@ bool ExecuteProgramBlocking(CString strApplication, CString strParameters, int *
         CloseHandle(ProcessInfo.hProcess);
         CloseHandle(ProcessInfo.hThread);
 
-        // let the windows expire (otherwise we commonly find a window and close it even though there was no reason)
-        Sleep(250);
-
-        // close any left-over windows (happens when FLAC fails and says there's an ID3v2 tag for example)
-        HWND hwndLeftover = FindWindow(APE_NULL, strWindowName);
-        if (hwndLeftover != APE_NULL)
-        {
-            // let the windows expire for longer
-            Sleep(1000);
-            hwndLeftover = FindWindow(APE_NULL, strWindowName);
-
-            if (hwndLeftover != APE_NULL)
-            {
-                // use the close window command instead of sending a WM_CLOSE
-                // for some reason doing the send was causing conversion of a pile of FLAC files to error because it was like a finishing encode was stopping a pending encode
-                // the window names should be unique so I couldn't figure out why (1/14/2026)
-                CloseWindow(hwndLeftover);
-                //SendMessage(hwndLeftover, WM_CLOSE, 0, 0);
-            }
-        }
+        // this used to find the window and destroy it if it was still around because FLAC left dangling windows when converting files with ID3v2 tags and failing
+        // but that no longer happens, so we just removed the code (1/16/2026)
+        // it could search for the window by using the unique name given it and destroy it that way, but the process has finished so this should be unnecessary
 
         bRetVal = true;
     }
