@@ -4,7 +4,7 @@ CAPEInfo:
 **************************************************************************************************/
 #include "All.h"
 #include "APEInfo.h"
-#include "IO.h"
+#include "IAPEIO.h"
 #include "APEHeader.h"
 #include "GlobalFunctions.h"
 #include "WholeFileIO.h"
@@ -39,7 +39,6 @@ APE_FILE_INFO::APE_FILE_INFO()
     nDecompressedBitrate = 0;
     nJunkHeaderBytes = 0;
     nSeekTableElements = 0;
-    nMD5Invalid = 0;
 }
 
 APE_FILE_INFO::~APE_FILE_INFO()
@@ -49,7 +48,7 @@ APE_FILE_INFO::~APE_FILE_INFO()
 /**************************************************************************************************
 Construction
 **************************************************************************************************/
-CAPEInfo::CAPEInfo(int * pErrorCode, const wchar_t * pFilename, CAPETag * pTag, bool bAPL, bool bReadOnly, bool bAnalyzeTagNow, bool bReadWholeFile)
+CAPEInfo::CAPEInfo(int * pErrorCode, const str_utfn * pFilename, CAPETag * pTag, bool bAPL, bool bReadOnly, bool bAnalyzeTagNow, bool bReadWholeFile)
 {
     *pErrorCode = ERROR_SUCCESS;
     CloseFile();
@@ -58,7 +57,7 @@ CAPEInfo::CAPEInfo(int * pErrorCode, const wchar_t * pFilename, CAPETag * pTag,
     m_bAPL = bAPL;
 
     // open the file
-    m_spIO.Assign(CreateCIO());
+    m_spIO.Assign(CreateIAPEIO());
 
     *pErrorCode = m_spIO->Open(pFilename, bReadOnly);
     if (*pErrorCode != ERROR_SUCCESS)
@@ -77,7 +76,7 @@ CAPEInfo::CAPEInfo(int * pErrorCode, const wchar_t * pFilename, CAPETag * pTag,
         if (nSize < (APE_BYTES_IN_MEGABYTE * 200))
         {
             CWholeFileIO * pWholeFile = CreateWholeFileIO(m_spIO, nSize);
-            if (pWholeFile != NULL)
+            if (pWholeFile != APE_NULL)
             {
                 m_spIO.SetDelete(false);
                 m_spIO.Assign(pWholeFile);
@@ -98,7 +97,7 @@ CAPEInfo::CAPEInfo(int * pErrorCode, const wchar_t * pFilename, CAPETag * pTag,
     if (pTag == APE_NULL)
     {
         // we don't want to analyze right away for non-local files
-        // since a single I/O object is shared, we can't tag and read at the same time(i.e.in multiple threads)
+        // since a single I/O object is shared, we can't tag and read at the same time (i.e. in multiple threads)
         if (StringIsEqual(pFilename, L"http://", false, 7) ||
             StringIsEqual(pFilename, L"m01p://", false, 7) ||
             StringIsEqual(pFilename, L"https://", false, 8) ||
@@ -112,12 +111,9 @@ CAPEInfo::CAPEInfo(int * pErrorCode, const wchar_t * pFilename, CAPETag * pTag,
     {
         m_spAPETag.Assign(pTag);
     }
-
-    // update
-    CheckHeaderInformation();
 }
 
-CAPEInfo::CAPEInfo(int * pErrorCode, CIO * pIO, CAPETag * pTag)
+CAPEInfo::CAPEInfo(int * pErrorCode, IAPEIO * pIO, CAPETag * pTag)
 {
     m_bAPL = false;
     *pErrorCode = ERROR_SUCCESS;
@@ -138,9 +134,6 @@ CAPEInfo::CAPEInfo(int * pErrorCode, CIO * pIO, CAPETag * pTag)
         m_spAPETag.Assign(new CAPETag(m_spIO, true, GetCheckForID3v1()));
     else
         m_spAPETag.Assign(pTag);
-
-    // update
-    CheckHeaderInformation();
 }
 
 /**************************************************************************************************
@@ -158,10 +151,11 @@ int CAPEInfo::CloseFile()
 {
     m_spIO.Delete();
     m_APEFileInfo.spWaveHeaderData.Delete();
-    m_APEFileInfo.spSeekBitTable.Delete();
     m_APEFileInfo.spSeekByteTable64.Delete();
     m_APEFileInfo.spAPEDescriptor.Delete();
-
+#ifdef APE_BACKWARDS_COMPATIBILITY
+    m_APEFileInfo.spSeekBitTable.Delete();
+#endif
     m_spAPETag.Delete();
 
     // re-initialize variables
@@ -171,37 +165,6 @@ int CAPEInfo::CloseFile()
     return ERROR_SUCCESS;
 }
 
-/**************************************************************************************************
-Performs sanity checks on all of the header data.
-**************************************************************************************************/
-int CAPEInfo::CheckHeaderInformation()
-{
-    // Fixes a bug with MAC 3.99 where conversion from APE to APE could include the file tag
-    // as part of the WAV terminating data.  This sanity check fixes the problem.
-    if ((m_APEFileInfo.spAPEDescriptor != APE_NULL) &&
-        (m_APEFileInfo.spAPEDescriptor->nTerminatingDataBytes > 0))
-    {
-        int64 nFileBytes = m_spIO->GetSize();
-        if (nFileBytes > 0)
-        {
-            nFileBytes -= m_spAPETag->GetTagBytes();
-            nFileBytes -= m_APEFileInfo.spAPEDescriptor->nDescriptorBytes;
-            nFileBytes -= m_APEFileInfo.spAPEDescriptor->nHeaderBytes;
-            nFileBytes -= m_APEFileInfo.spAPEDescriptor->nSeekTableBytes;
-            nFileBytes -= m_APEFileInfo.spAPEDescriptor->nHeaderDataBytes;
-            nFileBytes -= m_APEFileInfo.spAPEDescriptor->nAPEFrameDataBytes;
-            if (nFileBytes < m_APEFileInfo.nWAVTerminatingBytes)
-            {
-                m_APEFileInfo.nMD5Invalid = true;
-                m_APEFileInfo.nWAVTerminatingBytes = static_cast<uint32>(nFileBytes);
-                m_APEFileInfo.spAPEDescriptor->nTerminatingDataBytes = static_cast<uint32>(nFileBytes);
-            }
-        }
-    }
-
-    return ERROR_SUCCESS;
-}
-
 /**************************************************************************************************
 Get the file information about the file
 **************************************************************************************************/
@@ -238,7 +201,7 @@ Primary query function
 **************************************************************************************************/
 int64 CAPEInfo::GetInfo(IAPEDecompress::APE_DECOMPRESS_FIELDS Field, int64 nParam1, int64 nParam2)
 {
-    int64 nResult = -1;
+    int64 nResult = ERROR_UNDEFINED;
 
     switch (Field)
     {
@@ -321,7 +284,7 @@ int64 CAPEInfo::GetInfo(IAPEDecompress::APE_DECOMPRESS_FIELDS Field, int64 nPara
         nResult = m_APEFileInfo.nDecompressedBitrate;
         break;
     case IAPEDecompress::APE_INFO_PEAK_LEVEL:
-        nResult = -1; // no longer supported
+        nResult = ERROR_UNDEFINED; // no longer supported
         break;
     case IAPEDecompress::APE_INFO_SEEK_BIT:
     {
@@ -335,7 +298,11 @@ int64 CAPEInfo::GetInfo(IAPEDecompress::APE_DECOMPRESS_FIELDS Field, int64 nPara
             if ((nFrame < 0) || (static_cast<uint32>(nFrame) >= m_APEFileInfo.nTotalFrames))
                 nResult = 0;
             else
+#ifdef APE_BACKWARDS_COMPATIBILITY
                 nResult = m_APEFileInfo.spSeekBitTable[nFrame];
+#else
+                nResult = 0;
+#endif
         }
         break;
     }
@@ -353,20 +320,20 @@ int64 CAPEInfo::GetInfo(IAPEDecompress::APE_DECOMPRESS_FIELDS Field, int64 nPara
         char * pBuffer = reinterpret_cast<char *>(nParam1);
         const int64 nMaxBytes = nParam2;
 
-        if (m_APEFileInfo.nFormatFlags & MAC_FORMAT_FLAG_CREATE_WAV_HEADER)
+        if (m_APEFileInfo.nFormatFlags & APE_FORMAT_FLAG_CREATE_WAV_HEADER)
         {
-            if (m_APEFileInfo.nWAVDataBytes >= (4 * APE_BYTES_IN_GIGABYTE))
+            if (m_APEFileInfo.nWAVDataBytes >= (APE_BYTES_IN_GIGABYTE * 4))
             {
                 if (static_cast<APE::int64>(sizeof(RF64_HEADER)) > nMaxBytes)
                 {
-                    nResult = -1;
+                    nResult = ERROR_UNDEFINED;
                 }
                 else
                 {
-                    WAVEFORMATEX wfeFormat; APE_CLEAR(wfeFormat); GetInfo(IAPEDecompress::APE_INFO_WAVEFORMATEX, POINTER_TO_INT64(&wfeFormat), 0);
+                    WAVEFORMATEX wfeFormat; GetInfo(IAPEDecompress::APE_INFO_WAVEFORMATEX, APE_POINTER_TO_INT64(&wfeFormat), 0);
                     RF64_HEADER WAVHeader; FillRF64Header(&WAVHeader, m_APEFileInfo.nWAVDataBytes, &wfeFormat);
                     memcpy(pBuffer, &WAVHeader, sizeof(RF64_HEADER));
-                    nResult = 0;
+                    nResult = ERROR_SUCCESS;
                 }
 
             }
@@ -374,15 +341,15 @@ int64 CAPEInfo::GetInfo(IAPEDecompress::APE_DECOMPRESS_FIELDS Field, int64 nPara
             {
                 if (static_cast<APE::int64>(sizeof(WAVE_HEADER)) > nMaxBytes)
                 {
-                    nResult = -1;
+                    nResult = ERROR_UNDEFINED;
                 }
                 else
                 {
-                    WAVEFORMATEX wfeFormat; APE_CLEAR(wfeFormat); GetInfo(IAPEDecompress::APE_INFO_WAVEFORMATEX, POINTER_TO_INT64(&wfeFormat), 0);
+                    WAVEFORMATEX wfeFormat; GetInfo(IAPEDecompress::APE_INFO_WAVEFORMATEX, APE_POINTER_TO_INT64(&wfeFormat), 0);
                     WAVE_HEADER WAVHeader; FillWaveHeader(&WAVHeader, static_cast<int64>(m_APEFileInfo.nWAVDataBytes), &wfeFormat,
                         static_cast<intn>(m_APEFileInfo.nWAVTerminatingBytes));
                     memcpy(pBuffer, &WAVHeader, sizeof(WAVE_HEADER));
-                    nResult = 0;
+                    nResult = ERROR_SUCCESS;
                 }
             }
         }
@@ -390,13 +357,13 @@ int64 CAPEInfo::GetInfo(IAPEDecompress::APE_DECOMPRESS_FIELDS Field, int64 nPara
         {
             if (m_APEFileInfo.nWAVHeaderBytes > nMaxBytes)
             {
-                nResult = -1;
+                nResult = ERROR_UNDEFINED;
             }
             else
             {
                 if ((m_APEFileInfo.spWaveHeaderData != APE_NULL) && (m_APEFileInfo.nWAVHeaderBytes > 0))
                     memcpy(pBuffer, m_APEFileInfo.spWaveHeaderData, static_cast<size_t>(m_APEFileInfo.nWAVHeaderBytes));
-                nResult = 0;
+                nResult = ERROR_SUCCESS;
             }
         }
         break;
@@ -408,7 +375,7 @@ int64 CAPEInfo::GetInfo(IAPEDecompress::APE_DECOMPRESS_FIELDS Field, int64 nPara
 
         if (m_APEFileInfo.nWAVTerminatingBytes > static_cast<uint32>(nMaxBytes))
         {
-            nResult = -1;
+            nResult = ERROR_UNDEFINED;
         }
         else
         {
@@ -425,7 +392,7 @@ int64 CAPEInfo::GetInfo(IAPEDecompress::APE_DECOMPRESS_FIELDS Field, int64 nPara
                 // restore the file pointer
                 m_spIO->Seek(nOriginalFileLocation, SeekFileBegin);
             }
-            nResult = 0;
+            nResult = ERROR_SUCCESS;
         }
         break;
     }
@@ -433,11 +400,11 @@ int64 CAPEInfo::GetInfo(IAPEDecompress::APE_DECOMPRESS_FIELDS Field, int64 nPara
     {
         WAVEFORMATEX * pWaveFormatEx = reinterpret_cast<WAVEFORMATEX *>(nParam1);
         FillWaveFormatEx(pWaveFormatEx, WAVE_FORMAT_PCM, m_APEFileInfo.nSampleRate, m_APEFileInfo.nBitsPerSample, m_APEFileInfo.nChannels);
-        nResult = 0;
+        nResult = ERROR_SUCCESS;
         break;
     }
     case IAPEDecompress::APE_INFO_IO_SOURCE:
-        nResult = POINTER_TO_INT64(m_spIO.GetPtr());
+        nResult = APE_POINTER_TO_INT64(m_spIO.GetPtr());
         break;
     case IAPEDecompress::APE_INFO_FRAME_BYTES:
     {
@@ -446,14 +413,23 @@ int64 CAPEInfo::GetInfo(IAPEDecompress::APE_DECOMPRESS_FIELDS Field, int64 nPara
         // bound-check the frame index
         if ((nFrame < 0) || (static_cast<uint32>(nFrame) >= m_APEFileInfo.nTotalFrames))
         {
-            nResult = -1;
+            nResult = ERROR_UNDEFINED;
         }
         else
         {
             if (static_cast<uint32>(nFrame) != (m_APEFileInfo.nTotalFrames - 1))
+            {
                 nResult = GetInfo(IAPEDecompress::APE_INFO_SEEK_BYTE, nFrame + 1) - GetInfo(IAPEDecompress::APE_INFO_SEEK_BYTE, nFrame);
+            }
             else
-                nResult = static_cast<int64>(m_spIO->GetSize() - static_cast<int64>(m_spAPETag->GetTagBytes()) - static_cast<int64>(m_APEFileInfo.nWAVTerminatingBytes) - static_cast<int64>(GetInfo(IAPEDecompress::APE_INFO_SEEK_BYTE, nFrame)));
+            {
+                // APL files have a tag on the APL but not the original file
+                nResult = static_cast<int64>(m_spIO->GetSize() - static_cast<int64>(m_APEFileInfo.nWAVTerminatingBytes) - static_cast<int64>(GetInfo(IAPEDecompress::APE_INFO_SEEK_BYTE, nFrame)));
+
+                // remove the tag (but not for APL files because they have a tag on the APL but not the original data file)
+                if (m_spAPETag->GetIOMatches(m_spIO))
+                    nResult -= static_cast<int64>(m_spAPETag->GetTagBytes());
+            }
         }
         break;
     }
@@ -464,7 +440,7 @@ int64 CAPEInfo::GetInfo(IAPEDecompress::APE_DECOMPRESS_FIELDS Field, int64 nPara
         // bound-check the frame index
         if ((nFrame < 0) || (static_cast<uint32>(nFrame) >= m_APEFileInfo.nTotalFrames))
         {
-            nResult = -1;
+            nResult = ERROR_UNDEFINED;
         }
         else
         {
@@ -476,13 +452,30 @@ int64 CAPEInfo::GetInfo(IAPEDecompress::APE_DECOMPRESS_FIELDS Field, int64 nPara
         break;
     }
     case IAPEDecompress::APE_INFO_TAG:
-        nResult = POINTER_TO_INT64(static_cast<IAPETag *>(m_spAPETag.GetPtr()));
+        nResult = APE_POINTER_TO_INT64(static_cast<IAPETag *>(m_spAPETag.GetPtr()));
         break;
     case IAPEDecompress::APE_INFO_APL:
         nResult = static_cast<int64>(m_bAPL);
         break;
+    case IAPEDecompress::APE_INFO_MD5:
+        if (m_APEFileInfo.spAPEDescriptor != APE_NULL)
+        {
+            char * pBuffer = reinterpret_cast<char *>(nParam1);
+            memcpy(pBuffer, m_APEFileInfo.spAPEDescriptor->cFileMD5, sizeof(m_APEFileInfo.spAPEDescriptor->cFileMD5));
+            nResult = ERROR_SUCCESS;
+        }
+        break;
+    case IAPEDecompress::APE_INFO_MD5_MATCHES:
+        nResult = ERROR_INVALID_CHECKSUM;
+        if (m_APEFileInfo.spAPEDescriptor != APE_NULL)
+        {
+            char * pBuffer = reinterpret_cast<char *>(nParam1);
+            if (memcmp(pBuffer, m_APEFileInfo.spAPEDescriptor->cFileMD5, 16) == 0)
+                nResult = ERROR_SUCCESS;
+        }
+        break;
     case IAPEDecompress::APE_INTERNAL_INFO:
-        nResult = POINTER_TO_INT64(&m_APEFileInfo);
+        nResult = APE_POINTER_TO_INT64(&m_APEFileInfo);
         break;
     case IAPEDecompress::APE_DECOMPRESS_CURRENT_BLOCK:
     case IAPEDecompress::APE_DECOMPRESS_CURRENT_MS:
@@ -491,6 +484,7 @@ int64 CAPEInfo::GetInfo(IAPEDecompress::APE_DECOMPRESS_FIELDS Field, int64 nPara
     case IAPEDecompress::APE_DECOMPRESS_CURRENT_BITRATE:
     case IAPEDecompress::APE_DECOMPRESS_AVERAGE_BITRATE:
     case IAPEDecompress::APE_DECOMPRESS_CURRENT_FRAME:
+        // all other conditions to prevent compiler warnings (4061, 4062, and Clang)
         break;
     }
 
