@@ -6,19 +6,18 @@ namespace APE
 /**************************************************************************************************
 Definitions
 **************************************************************************************************/
-class CIO;
+class IAPEIO;
 
 /**************************************************************************************************
 Read / Write from an IO source and return failure if the number of bytes specified
 isn't read or written
 **************************************************************************************************/
-int ReadSafe(CIO * pIO, void * pBuffer, int nBytes);
-intn WriteSafe(CIO * pIO, void * pBuffer, intn nBytes);
+int ReadSafe(IAPEIO * pIO, void * pBuffer, int nBytes);
 
 /**************************************************************************************************
 Checks for the existence of a file
 **************************************************************************************************/
-bool FileExists(const wchar_t * pFilename);
+bool FileExists(const str_utfn * pFilename);
 
 /**************************************************************************************************
 Allocate aligned memory
@@ -94,4 +93,9 @@ inline uint64 Switch8Bytes(uint64 nValue)
     #define ConvertU64LE(val) (val)
 #endif
 
+/**************************************************************************************************
+Get the number of threads from the specified number (handles automatic)
+**************************************************************************************************/
+int GetNumberThreads(int nThreads);
+
 }
